package com.css.simulation.resource.scheduler.consumer;


import api.common.pojo.constants.DictConstants;
import api.common.pojo.dto.ProjectMessageDTO;
import api.common.util.JsonUtil;
import com.css.simulation.resource.scheduler.mapper.ProjectMapper;
import com.css.simulation.resource.scheduler.mapper.SensorCameraMapper;
import com.css.simulation.resource.scheduler.mapper.SensorOgtMapper;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.pojo.dto.task.*;
import com.css.simulation.resource.scheduler.pojo.po.TaskPO;
import com.fasterxml.jackson.core.JsonProcessingException;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
@Slf4j
public class ProjectConsumer {

    @Autowired
    ProjectMapper projectMapper;
    @Autowired
    TaskMapper taskMapper;
    @Autowired
    SensorCameraMapper sensorCameraMapper;
    @Autowired
    SensorOgtMapper sensorOgtMapper;
    @Autowired
    private KafkaTemplate<String, String> kafkaTemplate;

    @KafkaListener(groupId = "simulation-resource-scheduler", topics = "test")
    public void testConsumer(ConsumerRecord<String, String> projectRecord) {
        System.out.println("------- 消费成功：" + projectRecord.value());
    }

    @KafkaListener(groupId = "simulation-resource-scheduler", topics = "manualProject")
    public void parseProject(ConsumerRecord<String, String> projectRecord) throws JsonProcessingException {
        //1 读取 kafka 的 project 信息
        String projectJson = projectRecord.value();

        ProjectMessageDTO projectMessageDTO = JsonUtil.jsonToBean(projectJson, ProjectMessageDTO.class);
        String projectId = projectMessageDTO.getProjectId();    // 项目 id
        String taskMaxExecuteTime = projectMessageDTO.getTaskMaxExecuteTime();  // 任务最大执行时间
        String parallelism = projectMessageDTO.getParallelism();    // 并行度
        //2 修改该 project 的状态为执行中
        projectMapper.updateProjectState(projectId, "2");
        //3 根据 project 信息获取 task 信息。task 信息包括算法、模型、场景数据包等
        List<TaskPO> taskPOList = taskMapper.selectByProjectId(projectId);
        //4 组装 task 消息
        for (TaskPO taskPO : taskPOList) {
            String resultPath = "/project/manual-project/" + projectId + "/" + taskPO.getId();
            String scenarioOsc;
            String scenarioOdr;
            String scenarioOsgb;
            if (DictConstants.SCENE_NATURAL.equals(taskPO.getSceneType())) {
                scenarioOsc = taskPO.getSnScenarioOsc();
                scenarioOdr = taskPO.getSnScenarioOsc();
                scenarioOsgb = taskPO.getSnScenarioOsgb();
            } else if (DictConstants.SCENE_STANDARD.equals(taskPO.getSceneType())) {
                scenarioOsc = taskPO.getScrScenarioOsc();
                scenarioOdr = taskPO.getScrScenarioOsc();
                scenarioOsgb = taskPO.getScrScenarioOsgb();
            } else if (DictConstants.SCENE_ACCIDENT.equals(taskPO.getSceneType())) {
                scenarioOsc = taskPO.getSaScenarioOsc();
                scenarioOdr = taskPO.getSaScenarioOsc();
                scenarioOsgb = taskPO.getSaScenarioOsgb();
            } else {
                log.error("------- 不存在场景类型：" + taskPO.getSceneType());
                return;
            }
            String vehicleId = taskPO.getVehicleId();

            //4-1 组装 task 消息
            TaskDTO taskDTO = TaskDTO.builder()
                    .info(InfoDTO.builder()
                            .project_id(projectId)
                            .task_id(taskPO.getId())
                            .task_path(resultPath)
                            .build())
                    .scenario(ScenarioDTO.builder()
                            .scenario_osc(scenarioOsc)
                            .scenario_odr(scenarioOdr)
                            .scenario_osgb(scenarioOsgb)
                            .build())
                    .vehicle(VehicleDTO.builder()
                            .model(ModelDTO.builder()
                                    .model_label(taskPO.getModelLabel())
                                    .build())
                            .dynamics(DynamicsDTO.builder()
                                    .dynamics_maxspeed(taskPO.getMaxSpeed())
                                    .dynamics_enginepower(taskPO.getEnginePower())
                                    .dynamics_maxdecel(taskPO.getMaxDeceleration())
                                    .dynamics_maxsteering(taskPO.getMaxSteeringAngle())
                                    .dynamics_mass(taskPO.getMass())
                                    .dynamics_frontsurfaceeffective(taskPO.getFrontSurfaceEffective())
                                    .dynamics_airdragcoefficient(taskPO.getAirDragCoefficient())
                                    .dynamics_rollingresistance(taskPO.getRollingResistanceCoefficient())
                                    .dynamics_wheeldiameter(taskPO.getWheelDiameter())
                                    .dynamics_wheeldrive(taskPO.getWheelDrive())
                                    .dynamics_overallefficiency(taskPO.getOverallEfficiency())
                                    .dynamics_distfront(taskPO.getFrontDistance())
                                    .dynamics_distrear(taskPO.getRearDistance())
                                    .dynamics_distleft(taskPO.getLeftDistance())
                                    .dynamics_distright(taskPO.getRightDistance())
                                    .dynamics_distheight(taskPO.getHeightDistance())
                                    .dynamics_wheelbase(taskPO.getWheelbase())
                                    .build())
                            .sensors(SensorsDTO.builder()   // 根据 vehicleId 查询绑定的传感器列表
                                    .camera(sensorCameraMapper.selectCameraByVehicleId(taskPO.getVehicleId()))
                                    .OGT(sensorOgtMapper.selectOgtByVehicleId(taskPO.getVehicleId()))
                                    .build())
                            .build())
                    .build();
            //4-4 将对象转成 json
            String taskJson = JsonUtil.beanToJson(taskDTO);
            //4-5 将 projectId 作为 topic 名称，发送 task 信息到 kafka
            kafkaTemplate.send(projectId, 0, taskPO.getId(), taskJson);
        }
        //4 创建 pod 开始执行。

    }
}
